@@ -1,18 +1,25 @@
 /*
- * Copyright (c) 2011, Vicent Marti
+ * Copyright (c) 2015, Vicent Marti
  *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
- * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
- * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
- * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
- * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
- * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+ * THE SOFTWARE.
  */
+
 #include "redcarpet.h"
 
 VALUE rb_mRedcarpet;
@@ -79,7 +86,7 @@ rb_redcarpet_md__free(void *markdown)
 
 static VALUE rb_redcarpet_md__new(int argc, VALUE *argv, VALUE klass)
 {
-	VALUE rb_markdown, rb_rndr, hash;
+	VALUE rb_markdown, rb_rndr, hash, rndr_options;
 	unsigned int extensions = 0;
 
 	struct rb_redcarpet_rndr *rndr;
@@ -96,6 +103,12 @@ static VALUE rb_redcarpet_md__new(int argc, VALUE *argv, VALUE klass)
 
 	Data_Get_Struct(rb_rndr, struct rb_redcarpet_rndr, rndr);
 
+	/* Merge the current options in the @options hash */
+	if (hash != Qnil) {
+		rndr_options = rb_iv_get(rb_rndr, "@options");
+		rb_funcall(rndr_options, rb_intern("merge!"), 1, hash);
+	}
+
 	markdown = sd_markdown_new(extensions, 16, &rndr->callbacks, &rndr->options);
 	if (!markdown)
 		rb_raise(rb_eRuntimeError, "Failed to create new Renderer class");
@@ -122,13 +135,9 @@ static VALUE rb_redcarpet_md_render(VALUE self, VALUE text)
 	if (NIL_P(text))
 		return Qnil;
 
-#ifdef HAVE_RUBY_ENCODING_H
-	{
-		struct rb_redcarpet_rndr *renderer;
-		Data_Get_Struct(rb_rndr, struct rb_redcarpet_rndr, renderer);
-		renderer->options.active_enc = rb_enc_get(text);
-	}
-#endif
+	struct rb_redcarpet_rndr *renderer;
+	Data_Get_Struct(rb_rndr, struct rb_redcarpet_rndr, renderer);
+	renderer->options.active_enc = rb_enc_get(text);
 
 	/* initialize buffers */
 	output_buf = bufnew(128);
